"""Version package.

Checks version number for upgrades in PyPI
"""
from functools import lru_cache
from typing import List, Optional

import arrow
import requests
from loguru import logger
from pkg_resources import parse_version
from requests import RequestException

SUBDIVISIONS_PYPI_URL = "https://pypi.python.org/pypi/subdivisions/json"


@lru_cache()
def get_pypi_versions(time) -> Optional[List[str]]:
    """Return the version list data from PyPI.

    :return: list
    """

    versions_list = None
    try:
        ret = requests.get(SUBDIVISIONS_PYPI_URL, timeout=1)
        data = ret.json()
    except RequestException:
        return None
    if data:
        versions_list = list(data["releases"].keys())
        versions_list.sort(key=parse_version)
    return versions_list


def check_versions(verbose: bool = False) -> None:
    """Check if it is the latest version.

    Compares actual version vs last known
    version in PyPI, for upgrades

    :return:
        string
    """
    from subdivisions import __version__

    logger.info(f"Starting Subdivisions. Installed Version: {__version__}")
    last_version = __version__
    if verbose:
        logger.info(f"Checking for latest version. Current version is: {__version__}")

    now = arrow.utcnow().ceil("hour").timestamp()
    all_versions = get_pypi_versions(now)
    if all_versions:
        last_version = all_versions[-1]
    if parse_version(last_version) > parse_version(__version__) and (
        "rc" not in last_version
        and "b" not in last_version
        and "dev" not in last_version
    ):
        logger.warning(
            f"You're running a outdated version. "
            f"Last Version: {last_version} - Please update."
        )
    elif verbose:
        logger.success("Success: You're using the latest version.")



from dataclasses import dataclass

from loguru import logger

from subdivisions.base import AWSClientMixin
from subdivisions.config import sub_config
from subdivisions.exceptions import PubSubException


@dataclass
class SubDivisionsBuilder(AWSClientMixin):
    topic: str

    def get_kms_key(self):
        response = self.get_client("kms").list_aliases()
        logger.debug(f"KMS Response: {response}")
        pubsub_keys = [
            alias["TargetKeyId"]
            for alias in response["Aliases"]
            if alias["AliasName"] == sub_config.pub_key
        ]
        if pubsub_keys:
            logger.debug(f"Default KMS KeyId found: {pubsub_keys[0]}")
            return pubsub_keys[0]
        raise PubSubException("Encryption Key not available.")



SQS_IAM = """
{
  "Id": "SQSEventsPolicy",
  "Version": "2012-10-17",
  "Statement": [
    {
      "Sid": "Allow_SQS_Services",
      "Action": "sqs:*",
      "Effect": "Allow",
      "Resource": "arn:aws:sqs:*",
      "Principal": {
        "AWS": "*"
      }
    }
  ]
}
"""
SNS_IAM = {
    "Version": "2008-10-17",
    "Id": "__default_policy_ID",
    "Statement": [
        {
            "Sid": "__default_statement_ID",
            "Effect": "Allow",
            "Principal": {"AWS": "*"},
            "Action": [
                "SNS:GetTopicAttributes",
                "SNS:SetTopicAttributes",
                "SNS:AddPermission",
                "SNS:RemovePermission",
                "SNS:DeleteTopic",
                "SNS:Subscribe",
                "SNS:ListSubscriptionsByTopic",
                "SNS:Publish",
                "SNS:Receive",
            ],
            "Resource": "",
        },
        {
            "Sid": "Enable Eventbridge Events",
            "Effect": "Allow",
            "Principal": {"Service": "events.amazonaws.com"},
            "Action": "sns:Publish",
            "Resource": "*",
        },
    ],
}



"""PubSub Events.

Declare here all PubSub topics which will be used.

These Topics will be created on AWS, when you send his first message.

"""
from enum import Enum


class UserEvents(Enum):
    USER_REGISTERED = "user_registered"
    USER_ACTIVATED = "user_activated"
    USER_LOGGED_IN = "user_logged_in"


class AccountEvents(Enum):
    BANK_ACCOUNT_REGISTERED = "bank_account_registered"
    AD_ACCOUNT_REGISTERED = "google_ad_account_registered"
    POLISHED_STATEMENT_TABLE = "polished_statement_table"
    STATEMENT_TABLE_UPDATED = "statement_table_updated"


class ProposalEvents(Enum):
    PROPOSAL_SELECTED = "proposal_selected"
    PROPOSAL_APPROVED = "proposal_approved"
    PROPOSAL_ACCEPTED = "proposal_accepted"
    PROPOSAL_INDICATIVE = "proposal_indicative"
    PROPOSAL_REFUSED = "proposal_refused"
    PROPOSAL_EXPIRED = "proposal_expired"
    PROPOSAL_STATUS_UPDATED = "prpposal_status_updated"


class CompanyEvents(Enum):
    COMPANY_REGISTERED = "company_registered"
    COMPANY_UPDATED = "company_updated"


class HubspotEvents(Enum):
    ORIGINATION_REGISTERED = "origination_registered"
    ORIGINATION = "origination"
    ADDITIONAL_DATA_SENT = "additional_data_sent"
    LEAD_SCORE = "lead_score"


class AdsEvents(Enum):
    ADS_ACCOUNT_CONNECTION = "ads_account_connection"


class CredentialEvents(Enum):
    NEW_TAX_ID_REGISTERED = "new_tax_id_registered"


class PartnerEvents(Enum):
    I_WANT_TO_BE_A_PARTNER = "i_want_to_be_a_partner"


class PartnershipEvents(Enum):
    ORDER_NEW = "order_new"
    ORDER_DENIED = "order_denied"
    ORDER_APPROVED = "order_approved"
    ORDER_INDICATIVE = "order_indicative"
    ORDER_USER_VERIFICATION = "order_user_verification"


class ClientManagerEvents(Enum):
    SHAREHOLDER_REGISTRATION = "shareholder_registration"
    GUARANTOR_REGISTRATION = "guarantor_registration"
    PROCURATOR_REGISTER = "procurator_register"
    EXECUTIVE_REGISTRATION = "executive_registration"
    PEP_FORM = "pep_form"


class DocumentationEvents(Enum):
    GENERATED_DOCUMENTATION = "generated_documentation"
    DOCUMENTATION_OK = "documentation_ok"
    DISBURSEMENT_MADE = "disbursement_made"
    CLIENT_PROCESS_APPROVED = "client_process_approved"
    CLIENT_PROCESS_WITH_PENDENCIES = "client_process_with_pendencies"
    UPLOAD_FIRST_DOCUMENT = "upload_first_document"
    MEMBERSHIP_DOCUMENT = "membership_document"
    START_ANALYSIS = "start_analysis"


class AssetEvents(Enum):
    CREATED_EXPECTED_CASHFLOW_ENTRY = "created_expected_cashflow_entry"
    CREATED_INSTRUMENT = "created_instrument"


class BankEvents(Enum):
    CREATED_BOLETO = "created_boleto"
    UPDATED_BOLETO = "updated_boleto"
    CREATED_BOLETO_PROCESSED = "created_boleto_processed"


class CercEvent(Enum):
    NEW_SCHEDULE = "new_schedule"
    UPDATE_SCHEDULE = "update_schedule"


class BillingEvents(Enum):
    PAYMENT_INSTALLMENT = "payment_installment"


class AlternativeCreditEvents(Enum):
    PRICING_REQUEST = "pricing_request"


class TermsEvents(Enum):
    TERMS_ACCEPTANCE = "terms_acceptance"


class ClientEvents(Enum):
    SUBSCRIBER = "subscriber"
    LEAD = "lead"



import json
from dataclasses import dataclass
from typing import Optional

import stringcase
from loguru import logger

from subdivisions.builders import SubDivisionsBuilder
from subdivisions.config import sub_config
from subdivisions.exceptions import PubSubException
from subdivisions.policies import SQS_IAM


@dataclass
class SubDivisionsSQSBuilder(SubDivisionsBuilder):
    queue_arn: Optional[str] = None
    queue_url: Optional[str] = None

    @property
    def queue_name(self):
        suffix = (
            sub_config.event_suffix
            if sub_config.event_suffix
            else sub_config.default_suffix
        )
        return (
            f"{sub_config.queue_prefix or sub_config.default_prefix}"
            f"_{stringcase.snakecase(sub_config.source_name)}"
            f"_{self.topic}"
            f"{suffix}"
        )

    def create_queue(
        self, is_dead_letter: bool = False, dead_letter_arn: Optional[str] = None
    ) -> "SubDivisionsSQSBuilder":

        queue_name = (
            f"dead_letter_{self.queue_name}" if is_dead_letter else self.queue_name
        )

        logger.debug(f"Creating queue: {queue_name}")

        queue_attributes = {"KmsMasterKeyId": self.get_kms_key(), "Policy": SQS_IAM}
        if dead_letter_arn:
            queue_attributes["RedrivePolicy"] = json.dumps(
                {
                    "deadLetterTargetArn": dead_letter_arn,
                    "maxReceiveCount": sub_config.queue_max_receive_count,
                }
            )

        create_queue_arguments = {
            "QueueName": queue_name,
            "Attributes": queue_attributes,
        }
        if sub_config.sqs_tags:
            create_queue_arguments["tags"] = sub_config.sqs_tags

        create_response = self.get_client("sqs").create_queue(**create_queue_arguments)
        logger.debug(f"Queue Create Response is: {create_response}")
        return self.get_queue_attributes(create_response["QueueUrl"])

    def get_queue_attributes(self, queue_url: str):
        attributes_response = self.get_client("sqs").get_queue_attributes(
            QueueUrl=queue_url, AttributeNames=["QueueArn"]
        )
        logger.debug(f"Queue Attributes Response is: {attributes_response}")
        self.queue_arn = attributes_response["Attributes"]["QueueArn"]
        self.queue_url = queue_url
        return self

    def get_queue(self, from_dead_letter: bool = False):

        queue_name = (
            f"dead_letter_{self.queue_name}" if from_dead_letter else self.queue_name
        )

        response_queues = self.get_client("sqs").list_queues(
            QueueNamePrefix=sub_config.queue_prefix, MaxResults=1000
        )
        queue_list = [
            queue_url
            for queue_url in response_queues.get("QueueUrls", [])
            if queue_name in queue_url
        ]
        if not queue_list:
            raise PubSubException(f"Queue {queue_name} not found.")

        queue_url = queue_list[0]
        return self.get_queue_attributes(queue_url)

    def queue_exists(self) -> bool:
        try:
            queue_url = self.get_queue().queue_url
            return queue_url is not None
        except PubSubException:
            return False



import json
from dataclasses import dataclass
from typing import Optional

import stringcase
from loguru import logger

from subdivisions.builders import SubDivisionsBuilder
from subdivisions.config import sub_config
from subdivisions.policies import SNS_IAM


@dataclass
class SubDivisionsSNSBuilder(SubDivisionsBuilder):
    sns_arn: Optional[str] = None

    def create_sns_topic(self) -> "SubDivisionsSNSBuilder":
        SNS_IAM["Statement"][0]["Resource"] = "arn:aws:sns:us-east-1:*:{}".format(
            self.topic
        )

        suffix = (
            sub_config.event_suffix
            if sub_config.event_suffix
            else sub_config.default_suffix
        )
        sns_topic_name = stringcase.titlecase(
            f"{sub_config.sns_prefix or sub_config.default_prefix}"
            f"{stringcase.titlecase(self.topic)}"
            f"{suffix}"
        ).replace(" ", "")

        create_topic_arguments = {
            "Name": sns_topic_name,
            "Attributes": {
                "KmsMasterKeyId": self.get_kms_key(),
                "Policy": json.dumps(SNS_IAM),
            },
        }
        if sub_config.sns_tags:
            create_topic_arguments["Tags"] = sub_config.sns_tags

        create_response = self.get_client("sns").create_topic(**create_topic_arguments)
        logger.debug(f"SNS Creation Response is: {create_response}")
        sns_topic_arn = create_response["TopicArn"]
        attributes_response = self.get_client("sns").get_topic_attributes(
            TopicArn=sns_topic_arn
        )
        logger.debug(f"SNS Attributes Response is: {attributes_response}")
        self.sns_arn = sns_topic_arn
        return self

    def subscribe_sns_topic(self, topic_queue_arn: str):
        subscribe_response = self.get_client("sns").subscribe(
            TopicArn=self.sns_arn,
            Protocol="sqs",
            Endpoint=topic_queue_arn,
        )
        logger.debug(f"SNS Subscription Response is: {subscribe_response}")
